*| [JavaECS](../../README.md) | [docs](../overview.md) | [entity](./dir.md) | EntityManager[]().md*
# Entity Manager
| **In this Section** |
|-|
| [About](#about) |
| [Implementation](#implementation) |
| [Constructors](#constructors) |
| [Methods](#methods) |
| [Fields](#fields) |
| [Examples](#examples)|
| [Notes](#notes)|

## About
Controls adding and removing entities, and registration and deregistration of components to specific entities.

## Implementation
Before attempting to use a system, it must be registered.

Registration requires a call to `registerSystem(String systemName, ECSSystem system)`, with params of the `String name` and a object reference of the [ECSSystem](./ECSSystem.md). Using the object reference, a system can be invoked at any time.

See [EntityManager.java](/BrychanD/JaveECS/src/branch/master/javaecs/src/main/java/nz/ac/massey/programming_project_159333_s1_2021/EntityManager.java)

## Constructors
``` java
public EntityManager(){}
```
The default constructor initialises [unusedEntities](#unusedEntities) to a new queue containing `1024` unique integers.

<br>

``` java
public EntityManager(int maxEntities){}
```
Initialises [unusedEntities](#unusedEntities) to a new queue containing `maxEntities` unique integers.

<br>

## Methods
### addEntity
``` java
public Integer addEntity();
```
Pops the next available entity index off the queue, and returns its value.

Returns `-1` if there is no remaining unused entities.

<br>

### getRegistrations
``` java
public BitSet getRegistrations(int entity);
```
Returns the `BitSet` of registrations the `entity` has

<br>

### registerComponent
``` java
public void registerComponent(int component, int entity);
```
Sets the bit `component` in [entityRegistrations](#entityRegistrations) for `entity` to `true`.

<br>

### removeEntity
``` java
public void removeEntity(int entity);
```
Adds the entity's index back to the unused queue, and clears the assigned registration bits
#### Note
>This function should not be called directly. Doing so will leave component data in memory, and will cause desynchronisation of the system registrations.

<br>

### unregisterComponent
``` java
public void unregisterComponent(int component, int entity);
```
Sets the bit `component` in [entityRegistrations](#entityRegistrations) for `entity` to `false`.

<br>

## Fields
### entityRegistrations
``` java
List<BitSet> entityRegistrations;
```

<br>

### unusedEntities
``` java
Queue<Integer> unusedEntities;
```

<br>


